
BRANCHES_PREFIXES = {
    'bugfix':           'Bugfix',
    'fix':              'Bugfix', 
    'documentation':    'Documentation', 
    'doc':              'Documentation',
    'feature':          'Feature', 
    'structure':        'Structure',
    'devops':           'CI/CD',
    'ci':               'CI/CD',
    'cd':               'CI/CD',
    'enhancement':      'Enhancement',
    'improve':          'Enhancement',
    'improvement':      'Enhancement'
}

CATEGORIES_ICONS = {
    'Bugfix':           ':fire:',
    'Documentation':    ':books:',
    'CI/CD':            ':rocket:',
    'Enhancement':      ':sparkles:',
    'Feature':          ':heavy_plus_sign:',
    'Structure':        ':hammer:',
    'Release':          ':package:'
}
